# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

from redis import Redis


class DataStore:
    def __init__(self, bed_logging: object, host: str):
        self.bed_logging = bed_logging

        self.r = Redis(host=host, decode_responses=True)

    def subscribe(self, channel: str = "controller"):
        sub = self.r.pubsub()
        sub.subscribe([channel])

        return sub

    def update(self, group: str, group_value: str):
        if self.r.exists(group):
            existing_group_value = self.r.get(group)

            if existing_group_value != group_value:
                self.r.set(group, str(group_value))

                self.bed_logging.log(
                    message="Changed {} from {} => {}".format(
                        group,
                        existing_group_value,
                        group_value,
                    )
                )
        else:
            self.r.set(group, str(group_value))

            self.bed_logging.log(
                message="Set {} => {}".format(
                    group,
                    group_value,
                )
            )



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

from fastapi import FastAPI
from pydantic import BaseModel
from starlette.responses import RedirectResponse


class UserAuth(BaseModel):
    corp_id: str
    email: str
    password: str


class Api(FastAPI):
    def __init__(self, config: dict, datastore: object):
        super().__init__(title="Mobile App API")

        self.config = config
        self.datastore = datastore

        @self.get("/")
        def api_home():
            return RedirectResponse(url="/docs")

        @self.post("/v2/user_auth")
        def api_v2_user_auth(user_auth: UserAuth):
            return self.config["api"]["v2_user_auth"]

        @self.get("/v2/user/1/subscribe/devices")
        def api_v2_subscribe_devices():
            if self.datastore.exists("bed:authorize_code") and self.datastore.exists(
                "bed:device_mac_address"
            ):
                authorize_code = self.datastore.get("bed:authorize_code")
                device_mac_address = self.datastore.get("bed:device_mac_address")

                devices = {
                    "list": [
                        self.config["api"]["v2_subscribe_devices"]
                        | {
                            "mac": device_mac_address,
                            "name": "{}{}".format("BASE", device_mac_address),
                            "authorize_code": authorize_code,
                        }
                    ],
                    "version": 32,
                }
            else:
                devices = {
                    "list": [],
                    "version": 32,
                }

            return devices



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import os

import yaml

config_path = f"{os.path.join(os.path.dirname(__file__))}/../config"


def get_config(file: str = f"{config_path}/controller_api.yml"):
    with open(file) as f:
        return yaml.safe_load(f.read())



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

import json


def process_commands(
    bed_logging: object,
    datastore: object,
    bed_socket: object,
):
    command_sub = datastore.subscribe()

    while True:
        try:
            for item in command_sub.listen():
                if item["type"] == "message":
                    data = json.loads(item["data"])

                    if "name" and "command" in data:
                        bed_socket.socket_send_data(data["command"])

                        bed_logging.log(
                            message="Sent to Bed (command: {}): {}".format(
                                data["name"],
                                data["command"],
                            )
                        )
        except RuntimeError as e:
            bed_logging.log(message=e)



# Copyright 2021 Trevor Lauder.
# SPDX-License-Identifier: MIT

from fastapi import FastAPI, HTTPException
from starlette.responses import RedirectResponse

from bn_adjustable_bed.controller_api.datastore import DataStore


class Api(FastAPI):
    def __init__(self, config: dict, datastore: DataStore):
        super().__init__(title="Command API")

        self.config = config
        self.datastore = datastore

        @self.get("/")
        def api_home():
            return RedirectResponse(url="/docs")

        @self.get("/config")
        def api_config():
            return self.config

        @self.put("/command")
        def api_command(name: str):
            if name in self.config["commands"] and self.config["commands"][name] != "":
                self.datastore.publish_command(name=name)

                return {"name": name, "success": True}
            else:
                raise HTTPException(
                    status_code=404, detail={"name": name, "success": False}
                )
